/*
 * Copyright (c) 1991-2023 by STEP Tools Inc. 
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify, and distribute this software and
 * its documentation is hereby granted, provided that this copyright
 * notice and license appear on all copies of the software.
 * 
 * STEP TOOLS MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE
 * SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
 * BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT. STEP TOOLS
 * SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A
 * RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
 * DERIVATIVES.
 *
 * Author: David Loffredo (loffredo@steptools.com)
 */

#include <stp_schema.h>
#include "toolpath.h"

//------------------------------------------------------------
// Construct measure values and associated contexts
//
stp_measure_representation_item * ToolpathMaker::make_length_measure(
    const char * label,
    double val
    )
{
    if (!f_design) return 0;

    // Use a special complex instance for length measures
    stp_named_unit * u = make_length_unit();
    stp_measure_representation_item * mwu = 
	pnewIn(f_design) stp_length_measure_with_unit_and_measure_representation_item;

    mwu-> name (label? label: "");
    mwu-> value_component (pnewIn(f_design) stp_measure_value);
    mwu-> value_component()-> _length_measure(val);

    mwu-> unit_component (pnewIn(f_design) stp_unit);
    mwu-> unit_component()-> _named_unit(u);

    return mwu;
}

stp_measure_representation_item * ToolpathMaker::make_angle_measure(
    const char * label,
    double val
    )
{
    if (!f_design) return 0;

    // Use a special complex instance for plane angle measures
    stp_named_unit * u = make_angle_unit();
    stp_measure_representation_item * mwu = 
	pnewIn(f_design) stp_measure_representation_item_and_plane_angle_measure_with_unit;

    mwu-> name (label? label: "");
    mwu-> value_component (pnewIn(f_design) stp_measure_value);
    mwu-> value_component()-> _plane_angle_measure(val);

    mwu-> unit_component (pnewIn(f_design) stp_unit);
    mwu-> unit_component()-> _named_unit(u);

    return mwu;
}


stp_measure_representation_item * ToolpathMaker::make_feed_measure(
    const char * label,
    double val
    )
{
    if (!f_design) return 0;

    stp_derived_unit * u = make_feed_unit();
    stp_measure_representation_item * mwu = 
	pnewIn(f_design) stp_measure_representation_item;

    mwu-> name (label? label: "");
    mwu-> value_component (pnewIn(f_design) stp_measure_value);
    mwu-> value_component()-> _numeric_measure(val);

    mwu-> unit_component (pnewIn(f_design) stp_unit);
    mwu-> unit_component()-> _derived_unit(u);

    return mwu;
}

stp_measure_representation_item * ToolpathMaker::make_spindle_measure(
    const char * label,
    double val
    )
{
    if (!f_design) return 0;

    stp_derived_unit * u = make_spindle_unit();
    stp_measure_representation_item * mwu = 
	pnewIn(f_design) stp_measure_representation_item;

    mwu-> name (label? label: "");
    mwu-> value_component (pnewIn(f_design) stp_measure_value);
    mwu-> value_component()-> _numeric_measure(val);

    mwu-> unit_component (pnewIn(f_design) stp_unit);
    mwu-> unit_component()-> _derived_unit(u);

    return mwu;
}



stp_representation_context * ToolpathMaker::make_simple_context()
{
    if (!f_design) return 0;

    RoseCursor objs;
    RoseObject * obj;

    /* The data set should only contain one plain representation
     * context.  There may be instances of the subtypes, but only
     * one of these.
     */
    objs.traverse (f_design);
    objs.exact (ROSE_DOMAIN(stp_representation_context));

    if (obj = objs.next()) {
	return ROSE_CAST(stp_representation_context,obj);
    }

    /* should the make get or create?  */
    stp_representation_context * rc = 
	pnewIn(f_design) stp_representation_context;
    
    rc-> context_identifier ("");
    rc-> context_type ("units not necessary");
    return rc;
}



stp_representation_context * ToolpathMaker::make_geometry_context()
{
    if (!f_design) return 0;

    // The geometric context information gives the number of dimensions
    // (usually 3), length, angle, and solid angle units, and potentially
    // a global uncertainty measure for comparing brep point locations.
    //
    // This information is held by a single representation context object
    // that is an instance of the following subtypes:
    //
    //    - geometric_representation_context
    //    - global_uncertainty_assigned_context
    //    - global_unit_assigned_context (if an uncertainty is given)
    //
    stp_geometric_representation_context_and_global_unit_assigned_context * ctx = 
	    pnewIn (f_design) stp_geometric_representation_context_and_global_unit_assigned_context;

    // Set the general geometric part of the context
    ctx-> context_identifier ("ID1");
    ctx-> context_type ("3D");
    ctx-> coordinate_space_dimension (3);

    // Set the global units part of the context.  Each of the units is
    // wrapped in an instance of the stp_unit select type.
    //
    stp_unit * u;

    u = pnewIn(f_design) stp_unit;
    u-> _named_unit (make_length_unit());
    ctx-> units()-> add (u);

    u = pnewIn(f_design) stp_unit;
    u-> _named_unit (make_angle_unit());
    ctx-> units()-> add (u);

    u = pnewIn(f_design) stp_unit;
    u-> _named_unit (make_steradian_unit());
    ctx-> units()-> add (u);

    return ctx;
}




//------------------------------------------------------------
// Properties and descriptive strings
//  
stp_action_property * ToolpathMaker::make_action_property (
    const char * nm, 
    stp_action_method * am,
    stp_representation * rep
    )
{
    if (!f_design) return 0;

    stp_action_property * ap = pnewIn(f_design) stp_action_property;
    ap-> name (nm);
    ap-> description ("milling");
    ap-> definition (pnewIn(f_design) stp_characterized_action_definition);
    ap-> definition()-> _action_method (am);

    stp_action_property_representation * apr = 
	pnewIn(f_design) stp_action_property_representation;

    apr-> name ("");
    apr-> description ("milling");
    apr-> property (ap);
    apr-> representation (rep);

    return ap;
}


stp_representation * ToolpathMaker::make_string_rep (
    const char * name
    )
{
    if (!f_design) return 0;

    // these are always shared, so just give the rep item and
    // representation names of 'constant' to emphasise the point
    //
    RoseStringObject shortcut = "__CONSTANT STRING-REP ";
    shortcut += name;

    RoseObject * obj = f_design-> findObject (shortcut);
    if (obj) return ROSE_CAST (stp_representation, obj);

    // Not found, create a new one
    stp_descriptive_representation_item * it = 
	pnewIn(f_design) stp_descriptive_representation_item;

    it-> name("constant");
    it-> description (name);

    stp_representation * rep = pnewIn(f_design) stp_representation;
    rep-> name ("constant");
    rep-> context_of_items (make_simple_context());
    rep-> items()-> add (it);

    f_design-> addName (shortcut, rep);  // save shortcut for future calls

    return rep;
}
